'' =================================================================================================
''
''   File....... hc-8_cylon.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" Williams, EFX-TEK
''               Copyright (c) 2011-2012 EFX-TEK
''               All Rights Reserved
''   E-mail..... teamefx@efx-tek.com
''   Started.... 
''   Updated.... 06 APR 2012
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000
  

con

  PCB_REV = "C"                                                 ' production boards

  RX1     = 31                                                  ' programming / debug port
  TX1     = 30
  
  SDA     = 29                                                  ' eeprom / i2c
  SCL     = 28

  OUT_EN  = 27                                                  ' output enable     
  
  SIO     = 26                                                  ' TTL serial io

  NC25    = 25                                                  ' not connected
  NC24    = 24
  
  RX2     = 23                                                  ' RS-485 interface
  RX2_EN  = 22
  TX2_EN  = 21
  TX2     = 20

  NC19    = 19
  
  R_LED   = 18                                                  ' bi-color LED
  G_LED   = 17

  NC16    = 16
  
  OUT7    = 15                                                  ' channel outputs
  OUT6    = 14   
  OUT5    = 13 
  OUT4    = 12
  OUT3    = 11
  OUT2    = 10   
  OUT1    =  9 
  OUT0    =  8
          
  OPT_BR  =  7                                                  ' option inputs
  OPT_A1  =  6   
  OPT_A0  =  5
  OPT_SM  =  4

  DMX_A8  =  3                                                  ' DMX address.8 input

  DO_165  =  2                                                  ' 74x165 IO
  CLK_165 =  1 
  LD_165  =  0


  ' for EFX-TEK uSD adapter module
  ' -- must open all option switches when installed

  SD_CS   =  OPT_BR                                       
  SD_DI   =  OPT_A1  
  SD_CLK  =  OPT_A0  
  SD_DO   =  OPT_SM  


con

   #0, NO, YES
   #0, OFF, RED, GRN

   #0, NOCON, RX, TX, RXTX

   #0, LSBFIRST, MSBFIRST
  

con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  term  : "fullduplexserial64"


pub main | t, ch 

  setup_io                                                      ' setup HC-8+ IO pins

  t := cnt                                                      ' synchronize timer
  repeat
    repeat ch from OUT0 to OUT6
      high(ch)                                                  ' light LED
      waitcnt(t += (125 * MS_001))                              ' hold for 125ms
      low(ch)                                                   ' LED off
    repeat ch from OUT7 to OUT1
      high(ch)                 
      waitcnt(t += (125 * MS_001))               
      low(ch)                  


con

  { ================================= }
  {                                   }
  {  S U P P O R T   R O U T I N E S  } 
  {                                   }
  { ================================= }


var

  long  lastscan                                                ' ttl/DMX input scan


pub pause(ms) | t

'' Delay program ms milliseconds

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)                                              ' delay must be > 0
    waitcnt(t += MS_001)

  
pub setup_io

  outa[OUT7..OUT0] := %0000_0000                                ' clear outputs
  dira[OUT7..OUT0] := %1111_1111                                ' output mode                                  
  low(OUT_EN)                                                   ' enable 74x245 

  high(LD_165)                                                  ' setup x165 io pins
  low(CLK_165)
  input(DO_165)
  
  set_rs485(NOCON)                                              ' RS-485 off
         

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin, refreshes output mode

  !outa[pin]
  dira[pin] := 1
  

pub input(pin)

'' Set pin to input mode and read back current state

  dira[pin] := 0

  return ina[pin]


pub set_rgled(state)

'' Sets R/G LED

  case state
    OFF:
      outa[R_LED..G_LED] := %00
      
    RED:
      outa[R_LED..G_LED] := %10
      
    GRN:
      outa[R_LED..G_LED] := %01
                        
  dira[R_LED..G_LED] := %11


pub set_rs485(state)

'' Sets RS-485 RX and TX enable pins

  case state
    NOCON : outa[RX2_EN..TX2_EN] := %10
    RX    : outa[RX2_EN..TX2_EN] := %00 
    TX    : outa[RX2_EN..TX2_EN] := %11   
    RXTX  : outa[RX2_EN..TX2_EN] := %01

  dira[RX2_EN..TX2_EN] := %11 


pub option_inputs

'' Returns 4-bit value of OPTION dipswitch inputs
'' -- "1" bit = ON

  dira[OPT_BR..OPT_SM] := %0000                                 ' force to input mode

  return !ina[OPT_BR..OPT_SM] & %1111                           ' return bits


pub ttl_inputs(rescan) 

'' Return TTL inputs status
'' -- updates global var "lastscan" if rescan is true

  if (rescan)
    lastscan := x165_in(16, MSBFIRST) | (ina[DMX_A8] << 16)     ' read in0..in7, dmx addresss
   
  return (lastscan & $FF)                                       ' return TTL input bits


pub dmx_address(rescan) 

'' Return DMX address switch setting
'' -- updates global var "lastscan" if rescan is true

  if (rescan)
    lastscan := x165_in(16, MSBFIRST) | (ina[DMX_A8] << 16)     ' read in0..in7, dmx addresss

  return (lastscan >> 8) & $01FF                                ' return DMX address bits
 

pub x165_in(bits, shiftmode) | work

'' Input value from 74x165(s)      
                                                                                          
  outa[LD_165] := 0                                             ' load inputs   
  outa[LD_165] := 1                                                                        
                                                                                          
  work := 0                                                     ' clear workspace
  bits := 1 #> bits <# 32                                       ' limit to legal value         
  repeat bits                                                   ' read n bits         
    work := (work << 1) | ina[DO_165]                           ' get new bit             
    outa[CLK_165] := 1                                          ' blip clock              
    outa[CLK_165] := 0  

  if (shiftmode == LSBFIRST)                                    ' LSBFIRST result?
    work ><= bits                                               ' reverse bits

  return work
  

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      